/**
 * \par Copyright (C), 2016-2018, Erobot
 * \class EBotLineFollower
 * \brief   Driver for eBot line follwer device.
 * @file    EbotLineFollower.cpp
 * @author  Erobot
 * @version V1.0.0
 * @date    2018/04/10
 * @brief   Driver for line follwer device.
 *
 * \par Copyright
 * This software is Copyright (C), 2016-2018, Erobot. Use is subject to license \n
 * conditions. The main licensing options available are GPL V2 or Commercial: \n
 *
 * \par Open Source Licensing GPL V2
 * This is the appropriate option if you want to share the source code of your \n
 * application with everyone you distribute it to, and you also want to give them \n
 * the right to share who uses it. If you wish to use this software under Open \n
 * Source Licensing, you must contribute all your source code to the open source \n
 * community in accordance with the GPL Version 2 when your application is \n
 * distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * \par Description
 * This file is a drive for Me line follwer device, It supports line follwer device
 * V2.2 provided by the Erobot. The line follwer used Infrared Tube to Use infrared
 * receiver and transmitter to detect the black line.
 *
 * \par Method List:
 *
 *    1. void EBotLineFollower::setpin(uint8_t Sensor1,uint8_t Sensor2,uint8_t Sensor3)
 *    2. uint8_t EBotLineFollower::readSensors(void)
 *    3. bool EBotLineFollower::readSensor1(void)
 *    4. bool EBotLineFollower::readSensor2(void)
 *    5. bool EBotLineFollower::readSensor3(void)
 *
 * \par History:
 * <pre>
 * `<Author>`         `<Time>`        `<Version>`        `<Descr>`
 * Blue God          2018/04/10        1.0.0            Rebuild the old lib.
 * </pre>
 *
 * @example LineFollowerTest.ino
 */

#include "EBotLineFollower.h"

/**
 * Alternate Constructor which can call your own function to map the line follwer device to arduino port,
 * it will assigned the input pin.
 * \param[in]
 *   Sensor1 - arduino port(should digital pin)
 * \param[in]
 *   Sensor2 - arduino port(should digital pin)
 * \param[in]
 *   Sensor3 - arduino port(should digital pin)
 */
EBotLineFollower::EBotLineFollower(uint8_t Sensor1,uint8_t Sensor2,uint8_t Sensor3)
{
  _Sensor1 = Sensor1;
  _Sensor2 = Sensor2;
  _Sensor3 = Sensor3;
  pinMode(_Sensor1,INPUT);
  pinMode(_Sensor2,INPUT);
  pinMode(_Sensor3,INPUT);
}

/**
 * \par Function
 *   setpin
 * \par Description
 *   Reset the line follwer device available PIN by its arduino port.
 * \param[in]
 *   Sensor1 - arduino port(should digital pin)
 * \param[in]
 *   Sensor2 - arduino port(should digital pin)
 * \par Output
 *   None
 * \return
 *   None
 * \par Others
 *   None
 */
void EBotLineFollower::setpin(uint8_t Sensor1,uint8_t Sensor2,uint8_t Sensor3)
{
  _Sensor1 = Sensor1;
  _Sensor2 = Sensor2;
  _Sensor3 = Sensor3;
  pinMode(_Sensor1,INPUT);
  pinMode(_Sensor2,INPUT);
  pinMode(_Sensor3,INPUT);
}

/**
 * \par Function
 *   readSensors
 * \par Description
 *   Get the sensors state.
 * \par Output
 *   None
 * \return
 *   (0x00)-S1_IN_S2_IN:   sensor1 and sensor2 are both inside of black line \n
 *   (0x01)-S1_IN_S2_OUT:  sensor1 is inside of black line and sensor2 is outside of black line \n
 *   (0x02)-S1_OUT_S2_IN:  sensor1 is outside of black line and sensor2 is inside of black line \n
 *   (0x03)-S1_OUT_S2_OUT: sensor1 and sensor2 are both outside of black line
 * \par Others
 *   None
 */
uint8_t EBotLineFollower::readSensors(void)
{
   static uint8_t state = S1_IN_S2_IN_S3_IN;

  uint8_t s1State = digitalRead(_Sensor1);
  uint8_t s2State = digitalRead(_Sensor2);
  uint8_t s3State = digitalRead(_Sensor3);

  state = ( (1 & s3State) << 2) | ((1 & s2State) << 1) | s1State;

  if (state == 0)
  {
    return(8);
  }
  return(state);
}
/**
 * \par Function
 *   getOffset
 * \par Description
 *   Get the sensors offset.
 * \par Output
 *   None
 * \return
 *   (0x00)-S1_IN_S2_IN:   sensor1 and sensor2 are both inside of black line \n
 *   (0x01)-S1_IN_S2_OUT:  sensor1 is inside of black line and sensor2 is outside of black line \n
 *   (0x02)-S1_OUT_S2_IN:  sensor1 is outside of black line and sensor2 is inside of black line \n
 *   (0x03)-S1_OUT_S2_OUT: sensor1 and sensor2 are both outside of black line
 * \par Others
 *   None
 */
int8_t EBotLineFollower::getOffset(void)
{
  int8_t offset_value;
  uint8_t state = S1_IN_S2_IN_S3_IN;

  bool s1State = digitalRead(_Sensor1);
  bool s2State = digitalRead(_Sensor2);
  bool s3State = digitalRead(_Sensor3);

  state = ( (1 & s3State) << 2) | ((1 & s2State) << 1) | s1State;
  _lastError = _Error;
  switch(state)
  {
    default:
      // _Error = _lastError;
      // offset_value = _Error;
     break;
  }

  return(state);
}
/**
 * \par Function
 *   readSensor1
 * \par Description
 *   Get the sensors1(left sensors) state.
 * \par Output
 *   None
 * \return
 *   0: sensor1 is inside of black line \n
 *   1: sensor1 is outside of black line
 * \par Others
 *   None
 */
bool EBotLineFollower::readSensor1(void)
{

  return digitalRead(_Sensor1);
}

/**
 * \par Function
 *   readSensor2
 * \par Description
 *   Get the sensors2(right sensors) state.
 * \par Output
 *   None
 * \return
 *   0: sensor1 is inside of black line \n
 *   1: sensor1 is outside of black line
 * \par Others
 *   None
 */
bool EBotLineFollower::readSensor2(void)
{
  return digitalRead(_Sensor2);
}

/**
 * \par Function
 *   readSensor3
 * \par Description
 *   Get the sensors3(right sensors) state.
 * \par Output
 *   None
 * \return
 *   0: sensor1 is inside of black line \n
 *   1: sensor1 is outside of black line
 * \par Others
 *   None
 */
bool EBotLineFollower::readSensor3(void)
{

  return digitalRead(_Sensor3);

}