/**
 * \par Copyright (C), 2016-2018, Erobot
 * \class ELineFollower
 * \brief   Driver for eBot line follwer device.
 * @file    EBotLineFollower.h
 * @author  Erobot
 * @version V1.0.0
 * @date    2018/04/10
 * @brief   Header for for EBotLineFollower.cpp module
 *
 * \par Copyright
 * This software is Copyright (C), 2016-2018, Erobot. Use is subject to license \n
 * conditions. The main licensing options available are GPL V2 or Commercial: \n
 *
 * \par Open Source Licensing GPL V2
 * This is the appropriate option if you want to share the source code of your \n
 * application with everyone you distribute it to, and you also want to give them \n
 * the right to share who uses it. If you wish to use this software under Open \n
 * Source Licensing, you must contribute all your source code to the open source \n
 * community in accordance with the GPL Version 2 when your application is \n
 * distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * \par Description
 * This file is a drive for Me line follwer device, It supports line follwer device
 * V2.2 provided by the Erobot. The line follwer used Infrared Tube to Use infrared
 * receiver and transmitter to detect the black line.
 *
 * \par Method List:
 *
 *    1. void EBotLineFollower::setpin(uint8_t Sensor1,uint8_t Sensor2)
 *    2. uint8_t EBotLineFollower::readSensors(void)
 *    3. bool EBotLineFollower::readSensor1(void)
 *    4. bool EBotLineFollower::readSensor2(void)
 *
 * \par History:
 * <pre>
 * `<Author>`         `<Time>`        `<Version>`        `<Descr>`
 * Blue God         2018/04/10          1.0.0            Rebuild the old lib.
 * </pre>
 *
 */
#ifndef EBotLineFollower_H
#define EBotLineFollower_H

#include <stdint.h>
#include <stdbool.h>
#include <Arduino.h>
#define S1_IN_S2_IN_S3_IN    (0x00)
#define S1_OUT_S2_IN_S3_IN   (0x01)
#define S1_IN_S2_OUT_S3_IN   (0x02)
#define S1_OUT_S2_OUT_S3_IN  (0x03)
#define S1_IN_S2_IN_S3_OUT   (0x04)
#define S1_OUT_S2_IN_S3_OUT  (0x05)
#define S1_IN_S2_OUT_S3_OUT  (0x06)
#define S1_OUT_S2_OUT_S3_OUT  (0x07)


/**
 * Class: EBotLineFollower
 * \par Description
 * Declaration of Class EBotLineFollower.
 */

class EBotLineFollower
{
public:
/**
 * Alternate Constructor which can call your own function to map the line follwer device to arduino port,
 * it will assigned the input pin.
 * \param[in]
 *   Sensor1 - arduino port(should digital pin)
 * \param[in]
 *   Sensor2 - arduino port(should digital pin)
 * \param[in]
 *   Sensor3 - arduino port(should digital pin)
 */
  EBotLineFollower(uint8_t Sensor1,uint8_t Sensor2,uint8_t Sensor3);
/**
 * \par Function
 *   setpin
 * \par Description
 *   Reset the line follwer device available PIN by its arduino port.
 * \param[in]
 *   Sensor1 - arduino port(should digital pin)
 * \param[in]
 *   Sensor2 - arduino port(should digital pin)
 * \par Output
 *   None
 * \return
 *   None
 * \par Others
 *   None
 */
  void setpin(uint8_t Sensor1,uint8_t Sensor2,uint8_t Sensor3);

/**
 * \par Function
 *   readSensors
 * \par Description
 *   Get the sensors state.
 * \par Output
 *   None
 * \return
 *   (0x00)-S1_IN_S2_IN:   sensor1 and sensor2 are both inside of black line \n
 *   (0x01)-S1_IN_S2_OUT:  sensor1 is inside of black line and sensor2 is outside of black line \n
 *   (0x02)-S1_OUT_S2_IN:  sensor1 is outside of black line and sensor2 is inside of black line \n
 *   (0x03)-S1_OUT_S2_OUT: sensor1 and sensor2 are both outside of black line
 * \par Others
 *   None
 */
  uint8_t readSensors(void);
/**
 * \par Function
 *   getOffset
 * \par Description
 *   Get the sensors offset.
 * \par Output
 *   None
 * \return
 *   (0x00)-S1_IN_S2_IN:   sensor1 and sensor2 are both inside of black line \n
 *   (0x01)-S1_IN_S2_OUT:  sensor1 is inside of black line and sensor2 is outside of black line \n
 *   (0x02)-S1_OUT_S2_IN:  sensor1 is outside of black line and sensor2 is inside of black line \n
 *   (0x03)-S1_OUT_S2_OUT: sensor1 and sensor2 are both outside of black line
 * \par Others
 *   None
 */
  int8_t getOffset(void);
/**
 * \par Function
 *   readSensor1
 * \par Description
 *   Get the sensors1(left sensors) state.
 * \par Output
 *   None
 * \return
 *   0: sensor1 is inside of black line \n
 *   1: sensor1 is outside of black line
 * \par Others
 *   None
 */
  bool readSensor1(void);

/**
 * \par Function
 *   readSensor2
 * \par Description
 *   Get the sensors2(right sensors) state.
 * \par Output
 *   None
 * \return
 *   0: sensor1 is inside of black line \n
 *   1: sensor1 is outside of black line
 * \par Others
 *   None
 */
  bool readSensor2(void);
  /**
 * \par Function
 *   readSensor3
 * \par Description
 *   Get the sensors3(right sensors) state.
 * \par Output
 *   None
 * \return
 *   0: sensor1 is inside of black line \n
 *   1: sensor1 is outside of black line
 * \par Others
 *   None
 */
  bool readSensor3(void);
private:
  volatile uint8_t  _Sensor1;
  volatile uint8_t  _Sensor2;
  volatile uint8_t  _Sensor3;
  volatile int8_t  _lastError,_Error;
};
#endif
